% This function takes two RGB images (mostly an original and estimate of
% the same RGB image) and compute the MSE and PSNR for the L, C1, and
% C2 components between the two images.
%
% Note 1: ORIG and ESTM should have the same image size
% Note 2: Output format 
%            [MSE_PSNR] = [MSE_L MSE_C1 MSE_C2 PSNR_L PSNR_C1 PSNR_C2]

function [MSE_PSNR] = calc_mse_psnr_total(ORIG, ESTM, frame)

% Determine dimensions of the image
S = size(ORIG); N1 = S(1); N2 = S(2);

% For ORIG image, extract L, C1, C2 components
L = 0.25*ORIG(:,:,1) + 0.5*ORIG(:,:,2) + 0.25*ORIG(:,:,3);
C1 = -0.25*ORIG(:,:,1) + 0.5*ORIG(:,:,2) -0.25*ORIG(:,:,3);
C2 = -0.25*ORIG(:,:,1) + 0.25*ORIG(:,:,3);

R = ORIG(:,:,1);
G = ORIG(:,:,2);
B = ORIG(:,:,3);

% For ESTM image, extract L, C1, C2 components
Lhat = 0.25*ESTM(:,:,1) + 0.5*ESTM(:,:,2) + 0.25*ESTM(:,:,3);
C1hat = -0.25*ESTM(:,:,1) + 0.5*ESTM(:,:,2) -0.25*ESTM(:,:,3);
C2hat = -0.25*ESTM(:,:,1) + 0.25*ESTM(:,:,3);

R_ESTM = ESTM(:,:,1);
G_ESTM = ESTM(:,:,2);
B_ESTM = ESTM(:,:,3);

% Compute MSE results
MSE_L = sum(sum((L(frame+1:N1-frame,frame+1:N2-frame)-...
    Lhat(frame+1:N1-frame,frame+1:N2-frame)).^2))*255^2/((N1-2*frame)*(N2-2*frame));
MSE_C1 = sum(sum((C1(frame+1:N1-frame,frame+1:N2-frame)-...
    C1hat(frame+1:N1-frame,frame+1:N2-frame)).^2))*255^2/((N1-2*frame)*(N2-2*frame));
MSE_C2 = sum(sum((C2(frame+1:N1-frame,frame+1:N2-frame)-...
    C2hat(frame+1:N1-frame,frame+1:N2-frame)).^2))*255^2/((N1-2*frame)*(N2-2*frame));

MSE_R = sum(sum((R(frame+1:N1-frame,frame+1:N2-frame)-...
    R_ESTM(frame+1:N1-frame,frame+1:N2-frame)).^2))*255^2/((N1-2*frame)*(N2-2*frame));
MSE_G = sum(sum((G(frame+1:N1-frame,frame+1:N2-frame)-...
    G_ESTM(frame+1:N1-frame,frame+1:N2-frame)).^2))*255^2/((N1-2*frame)*(N2-2*frame));
MSE_B = sum(sum((B(frame+1:N1-frame,frame+1:N2-frame)-...
    B_ESTM(frame+1:N1-frame,frame+1:N2-frame)).^2))*255^2/((N1-2*frame)*(N2-2*frame));

% Compute PSNR results
PSNR_L = 10*log10(255^2/MSE_L);
PSNR_C1 = 10*log10(255^2/MSE_C1);
PSNR_C2 = 10*log10(255^2/MSE_C2);
CMSE_LCC = (MSE_L + MSE_C1 + MSE_C2)/3;
CPSNR_LCC = 10*log10(255^2/CMSE_LCC);

PSNR_R = 10*log10(255^2/MSE_R);
PSNR_G = 10*log10(255^2/MSE_G);
PSNR_B = 10*log10(255^2/MSE_B);
CMSE_RGB=(MSE_R + MSE_G + MSE_B)/3;
CPSNR_RGB = 10*log10(255^2/CMSE_RGB);

scielab = s_cielab_result(uint8(ORIG*255),uint8(ESTM*255), frame);

% Packge the results
MSE_PSNR = [MSE_L MSE_C1 MSE_C2 CMSE_LCC PSNR_L PSNR_C1 PSNR_C2 CPSNR_LCC MSE_R MSE_G MSE_B CMSE_RGB PSNR_R PSNR_G PSNR_B CPSNR_RGB scielab];